;
; HUC_GFX.ASM  -  HuC Graphic Library
;

; ----
; local variables

          .zp
__spr_ptr .ds 2

          .bss
_satb    .ds 512	; the local SATB


; ----
; library code

	 .code

; vsync(char nb_frame)
; ----

_vsync:
	  txa
	  jmp	wait_vsync

; spr_set(char nb)
; ----
; load SI with the offset of the sprite to change
; SI = satb + 8 * sprite_number
; ----

_spr_set:
	  stz	<__spr_ptr+1
	  txa
	  asl	A
	  asl	A
	  asl	A
	  rol	<__spr_ptr+1
	  sta	<__spr_ptr
	  addw	#_satb,<__spr_ptr
	  rts

; spr_x(int value)
; ----

_spr_x:
	  ldy	#3
	  sta	[__spr_ptr],Y
	  txa
	  dey
	  sta	[__spr_ptr],Y
	  rts

; spr_y(int value)
; ----

_spr_y:
	  ldy	#1
	  sta	[__spr_ptr],Y
	  txa
	  sta	[__spr_ptr]
	  rts

; spr_pattern(int vaddr)
; ----

_spr_pattern:
	  sta	<__temp
	  txa
	  asl	A
	  rol	<__temp
	  rol	A
	  rol	<__temp
	  rol	A
	  rol	<__temp
	  rol	A
	  and	#$7
	  ldy	#5
	  sta	[__spr_ptr],Y
	  lda	<__temp
	  dey
	  sta	[__spr_ptr],Y
	  rts

; spr_ctrl(char mask, char value)
; ----

_spr_ctrl:
	  txa
	  and	[__sp]
	  sta	<__temp
	  lda	[__sp]
	  eor	#$FF
	  ldy	#7
	  and	[__spr_ptr],Y
	  ora	<__temp
	  sta	[__spr_ptr],Y
	  addw	#2,<__sp
	  rts

; spr_pal(char pal)
; ----

_spr_pal:
	  txa
	  and	#$0F
	  sta	<__temp
	  ldy	#6
	  lda	[__spr_ptr],Y
	  and	#$F0
	  ora	<__temp
	  sta	[__spr_ptr],Y
	  rts

; spr_pri(char pri)
; ----

_spr_pri:
	  ldy	#6
	  lda	[__spr_ptr],Y
	  and	#$7F
	  cpx	#$00
	  beq	.l1
	  ora	#$80
.l1:
	  sta	[__spr_ptr],Y
	  rts

; satb_update()
; ----

_satb_update:
	  update_satb _satb	; upload the SATB in VRAM
	  rts

; init_satb()
; ----

_init_satb:
	  init_satb _satb
	  rts

; get_color(int index)
; ----
; index: index in the palette (0-511)
; ----

_get_color:
	  stx	$0402
	  sta	$0403
	  ldx	$0404
	  lda	$0405
	  rts


; set_color(int index, int color)
; ----
; set one palette entry to the specified color
; ----
; index: index in the palette (0-511)
; color: color value,  GREEN: bit 6-8
;                      RED:   bit 3-5
;                      BLUE:  bit 0-2
; ----

_set_color:
	  tay
	  lda	[__sp]
	  sta	$0402
	  incw	<__sp
	  lda	[__sp]
	  sta	$0403
	  stx	$0404
	  sty	$0405
	  incw	<__sp
	  rts

; set_color_rgb(int index, char r, char g, char b)
; ----
; set one palette entry to the specified color
; ----
; index: index in the palette (0-511)
; r:     red              RED:   bit 3-5
; g:     green            GREEN: bit 6-8
; b:     blue             BLUE:  bit 0-2
; ----

_set_color_rgb:
	  ldy	#4
	  lda	[__sp],Y
	  sta	$0402
	  iny
	  lda	[__sp],Y
	  sta	$0403

	  txa
	  and	#$7
	  sta	<__temp
	  ldy	#2
	  lda	[__sp],Y
	  asl	A
	  asl	A
	  asl	A
	  ora	<__temp
	  asl	A
	  asl	A
	  sta	<__temp
	  lda	[__sp]
	  lsr	A
	  ror	<__temp
	  lsr	A
	  ror	<__temp
	  ldx	<__temp
	  stx	$0404
	  sta	$0405
	  addw	#6,<__sp
	  rts

; put_tile(int pattern, int position)
; ----
; draw a single tile at a given position
; ----
; pattern: vram address of the tile pattern
; position: position on screen where to put the tile
; ----
_put_tile:

        stx  <_di
        sta  <_di+1
        jsr  set_write

        ; addw #2, <__sp

        ; __ldwi 4
        ; jsr asr   ; shift the int on the stack (tile pattern address in Vram)
        ;          ; by 4 and even restore the stack position

        __ldwp __sp
        __stw video_data
        addw #2, <__sp
        rts

